@@ -9,6 +9,7 @@ from pdb import set_trace
 from cvnn.utils import create_folder
 import logging
 import cvnn
+from typing import Optional
 
 logger = logging.getLogger(cvnn.__name__)
 AVAILABLE_LIBRARIES = set()
@@ -23,7 +24,7 @@ except ImportError as e:
 try:
     import matplotlib.pyplot as plt
     AVAILABLE_LIBRARIES.add('matplotlib')
-    DEFAULT_MATPLOTLIB_COLORS = plt.rcParams['axes.prop_cycle'].by_key()['color']  # [1:] Uncomment to remove blue color
+    DEFAULT_MATPLOTLIB_COLORS = plt.rcParams['axes.prop_cycle'].by_key()['color'][1:] # Uncomment to remove blue color
 except ImportError as e:
     logger.info("Matplotlib not installed, consider installing it to get more plotting capabilities")
 if 'matplotlib' in AVAILABLE_LIBRARIES:
@@ -177,6 +178,19 @@ def get_trailing_number(s):
     return int(os.path.splitext(m.group())[0]) if m else None
 
 
+def get_data_files_list(root_path):
+    """
+    Reads all files inside the root file (and sub-folders) and returns the list of those that end with TODO
+    :return: The list of all files
+    """
+    data_files = []
+    for path, subdirs, files in os.walk(root_path):
+        for name in files:
+            if name.endswith("run_data.csv"):
+                data_files.append(os.path.join(path, name))
+    return data_files
+
+
 # ----------------
 # Confusion Matrix
 # ----------------
@@ -253,34 +267,37 @@ def get_confusion_matrix(y_pred_np, y_label_np, filename=None, axis_legends=None
 
 
 class SeveralMonteCarloComparison:
+    """
+    This class is used to compare several monte carlo runs done with cvnn.montecarlo.MonteCarlo class.
+    MonteCarlo let's you compare different models between them but let's you not change other values like epochs.
+    You can run as several MonteCarlo runs and then use SeveralMonteCarloComparison class to compare the results.
+
+    Example of usage:
+
+    ```
+    paths = []
+    # Run several Monte Carlo's
+    for learning_rate in learning_rates:
+        monte_carlo = RealVsComplex(complex_network)
+        result_path = monte_carlo.run(x, y, iterations=iterations, learning_rate=learning_rate,
+                        epochs=epochs, batch_size=batch_size, display_freq=display_freq,
+                        shuffle=True, debug=debug, data_summary=dataset.summary())
+        paths.append(result_path)
+    # paths will look something like ["path/to/1st/run/run_data",
+    #                                 "path/to/2nd/run/run_data",
+    #                                 "path/to/3rd/run/run_data",
+    #                                 "path/to/4th/run/run_data"]
+    # Run self
+    several = SeveralMonteCarloComparison('learning rate', x = learning_rates, paths =paths)
+    several.box_plot(showfig=True)
+    ```
+    """
 
     def __init__(self, label, x, paths, round=2):
         """
-        This class is used to compare several monte carlo runs done with cvnn.montecarlo.MonteCarlo class.
-        MonteCarlo let's you compare different models between them but let's you not change other values like epochs.
-        You can run as several MonteCarlo runs and then use SeveralMonteCarloComparison class to compare the results.
-
-        Example of usage:
-
-        ```
-        # Run several Monte Carlo's
-        for learning_rate in learning_rates:
-            monte_carlo = RealVsComplex(complex_network)
-            monte_carlo.run(x, y, iterations=iterations, learning_rate=learning_rate,
-                            epochs=epochs, batch_size=batch_size, display_freq=display_freq,
-                            shuffle=True, debug=debug, data_summary=dataset.summary())
-        # Run self
-        several = SeveralMonteCarloComparison('learning rate', x = learning_rates,
-                                              paths = ["path/to/1st/run/run_data",
-                                                       "path/to/2nd/run/run_data",
-                                                       "path/to/3rd/run/run_data",
-                                                       "path/to/4th/run/run_data"]
-        several.box_plot(showfig=True)
-        ```
-
-        :label: string that describes what changed between each montecarlo run
-        :x: List of the value for each monte carlo run wrt :label:.
-        :paths: Full path to each monte carlo run_data saved file (Must end with run_data)
+        :param label: string that describes what changed between each montecarlo run
+        :param x: List of the value for each monte carlo run wrt :label:.
+        :param paths: Full path to each monte carlo run_data saved file (Must end with run_data)
             NOTE: x and paths must be the same size
         """
         self.x_label = label
@@ -304,7 +321,18 @@ class SeveralMonteCarloComparison:
         self.df = pd.concat(frames)
         self.monte_carlo_analyzer = MonteCarloAnalyzer(df=self.df)
 
-    def box_plot(self, key='accuracy', library='plotly', epoch=-1, showfig=False, savefile=None):
+    def box_plot(self, key='accuracy', library='plotly', epoch=-1, showfig=False, savefile: Optional[str] = None):
+        """
+        Saves/shows a box plot of the results.
+
+        :param key: String stating what to plot using tf.keras.History labels. ex. `val_accuracy` for the validation acc
+        :param library: string stating the library to be used to generate the box plot.
+            - `plotly <https://plotly.com/python/>`_
+            - `seaborn <https://seaborn.pydata.org/>`_
+        :param epoch: Which epoch to use for the box plot. If -1 (default) it will use the last epoch.
+        :param showfig: If True, it will show the grated box plot
+        :param savefile: String with the path + filename where to save the boxplot. If None (default) no figure is saved
+        """
         if library == 'plotly':
             self._box_plot_plotly(key=key, epoch=epoch, showfig=showfig, savefile=savefile)
         # TODO: https://seaborn.pydata.org/examples/grouped_boxplot.html
@@ -458,7 +486,8 @@ class SeveralMonteCarloComparison:
 
 class Plotter:
 
-    def __init__(self, path, file_suffix:str = "_results_fit.csv", data_results_dict: dict = None, model_name: str = None):
+    def __init__(self, path, file_suffix: str = "_results_fit.csv",
+                 data_results_dict: dict = None, model_name: str = None):
         """
         This class manages the plot of results for a model train.
         It opens the csv files (test and train) saved during training and plots results as wrt each epoch saved.
@@ -475,14 +504,12 @@ class Plotter:
         if not file_suffix.endswith(".csv"):
             file_suffix += ".csv"
         self.file_suffix = file_suffix
-        self.pandas_list = []
-        self.labels = []
         self.model_name = model_name
+        self.pandas_dict = {}
         
         if data_results_dict:
             result_pandas = pd.DataFrame.from_dict(data_results_dict)
             result_pandas.to_csv(self.path / f"{model_name}{file_suffix}", index=False)
-        
         self._csv_to_pandas()
 
     def _csv_to_pandas(self):
@@ -491,17 +518,22 @@ class Plotter:
         Also saves the name of the file where it got the pandas frame as a label.
         This function is called by the constructor.
         """
-        self.pandas_list = []
-        self.labels = []
-        files = os.listdir(self.path)
-        files.sort()  # Respect the colors for the plot of Monte Carlo.
+        self.pandas_dict = {}
+        # files = os.listdir(self.path)  
         # For ComplexVsReal Monte Carlo it has first the Complex model and SECOND the real one.
         # So ordering the files makes sure I open the Complex model first and so it plots with the same colours.
         # TODO: Think a better way without loosing generality (This sort is all done because of the ComplexVsReal case)
-        for file in files:
-            if file.endswith(self.file_suffix):
-                self.pandas_list.append(pd.read_csv(self.path / file))
-                self.labels.append(re.sub(self.file_suffix + '$', '', file).replace('_', ' '))
+        for path, subdirs, files in os.walk(self.path):
+            files.sort()  # Respect the colors for the plot of Monte Carlo.
+            for file in files:
+                if file.endswith(self.file_suffix):
+                    label = re.sub(self.file_suffix + '$', '', file).replace('_', ' ')
+                    tmp_df = pd.read_csv(Path(path) / file)
+                    if self.pandas_dict.get(label) is None:
+                        self.pandas_dict[label] = tmp_df
+                    else:
+                        self.pandas_dict[label] = pd.concat([self.pandas_dict[label], tmp_df], ignore_index=True)
+        # set_trace()
 
     def reload_data(self):
         """
@@ -521,13 +553,13 @@ class Plotter:
         """
         # https://pandas.pydata.org/pandas-docs/stable/user_guide/merging.html
         self._csv_to_pandas()
-        if len(self.pandas_list) == 0:
-            logger.error("Error: There was no csv logs to open")
-            sys.exit(-1)
-        length = len(self.pandas_list[0])
-        for data_frame in self.pandas_list:  # TODO: Check if.
+        if not bool(self.pandas_dict):
+            # logger.error("Error: There was no csv logs to open")
+            raise IOError("There was no csv logs to open")
+        length = len(next(iter(self.pandas_dict.values())))
+        for data_frame in self.pandas_dict.values():  # TODO: Check if.
             if not length == len(data_frame):  # What happens if NaN? Can I cope not having same len?
-                logger.error("Data frame length should have been {0} and was {1}".format(length, len(data_frame)))
+                raise ValueError("Data frame length should have been {0} and was {1}".format(length, len(data_frame)))
 
         result = pd.DataFrame({
             'network': [self.get_net_name()] * length,
@@ -535,7 +567,7 @@ class Plotter:
             'path': [self.path] * length
         })
 
-        for data_frame, data_label in zip(self.pandas_list, self.labels):
+        for data_label, data_frame in self.pandas_dict.items():
             # data_frame.columns = [data_label + " " + str(col) for col in data_frame.columns]
             # concatenated = pd.concat(self.pandas_list, keys=self.labels)
             result = pd.concat([result, data_frame], axis=1, sort=False)
@@ -559,7 +591,7 @@ class Plotter:
                         extension=".svg"):
         if reload:
             self._csv_to_pandas()
-        if not len(self.pandas_list) != 0:
+        if not len(self.pandas_dict) != 0:
             logger.error("Empty pandas list to plot")
             return None
         for key in ["loss", "accuracy"]:
@@ -568,6 +600,17 @@ class Plotter:
 
     def plot_key(self, key='loss', reload=False, library='plotly', showfig=False, savefig=True, index_loc=None,
                  extension=".svg"):
+        """
+        :param library: string stating the library to be used to generate the box plot.
+            - `matplotlib <https://matplotlib.org/stable/index.html>`_
+            - `plotly <https://plotly.com/python/>`_
+        :param key: String stating what to plot using tf.keras.History labels. ex. `val_accuracy` for the validation acc
+        :param showfig: If True, it will show the grated box plot
+        :param savefig: If True, it saves the figure at `self.path/key_library.extension`
+        :param reload: If True it will reload data from the csv file in case it has changed.
+        :param index_loc:
+        :param extension: file extensions (default svg) to be used when saving the file (ignored if library is plotly).
+        """
         if reload:
             self._csv_to_pandas()
         if library == 'matplotlib':
@@ -585,11 +628,11 @@ class Plotter:
         fig, ax = plt.subplots()
         ax.set_prop_cycle('color', DEFAULT_MATPLOTLIB_COLORS)
         title = None
-        for i, data in enumerate(self.pandas_list):
+        for label, data in self.pandas_dict.items():
             if title is not None:
-                title += " vs. " + self.labels[i]
+                title += " vs. " + label
             else:
-                title = self.labels[i]
+                title = label
             for k in data:
                 if key in k:
                     if index_loc is not None:
@@ -597,7 +640,7 @@ class Plotter:
                             data = data[data['stats'] == 'mean']
                         else:
                             logger.warning("Warning: Trying to index an array without index")
-                    ax.plot(data[k], 'o-', label=(k.replace(key, '') + self.labels[i]).replace('_', ' '))
+                    ax.plot(data[k], 'o-', label=(k.replace(key, '') + label).replace('_', ' '))
         title += " " + key
         fig.legend(loc="upper right")
         ax.set_ylabel(key)
@@ -616,15 +659,17 @@ class Plotter:
         fig = go.Figure()
         annotations = []
         title = None
-        for i, data in enumerate(self.pandas_list):
+        i = 0
+        for label, data in self.pandas_dict.items():
+            i += 1
             if title is not None:
-                title += " vs. " + self.labels[i]
+                title += " vs. " + label
             else:
-                title = self.labels[i]
+                title = label
             j = 0
             for k in data:
                 if key in k:
-                    color = DEFAULT_PLOTLY_COLORS[j * len(self.pandas_list) + i]
+                    color = DEFAULT_PLOTLY_COLORS[j * len(self.pandas_dict.values()) + i]
                     j += 1
                     if index_loc is not None:
                         if 'stats' in data.keys():
@@ -633,7 +678,7 @@ class Plotter:
                             logger.warning("Trying to index an array without index")
                     x = list(range(len(data[k])))
                     fig.add_trace(go.Scatter(x=x, y=data[k], mode='lines',
-                                             name=(k.replace(key, '') + self.labels[i]).replace('_', ' '),
+                                             name=(k.replace(key, '') + label).replace('_', ' '),
                                              line_color=color))
                     # Add points
                     fig.add_trace(go.Scatter(x=[x[-1]],
@@ -682,7 +727,20 @@ class Plotter:
 
 class MonteCarloPlotter(Plotter):
 
+    """
+    Inherits from Plotter.
+    Used by cvnn.data_analysis.MonteCarloAnalyzer.
+
+    As cvnn.data_analysis.Plotter, this class is used to plot evolution graphs with respect to each epoch like loss
+        or accuracy but uses the average and/or median result of a monte carlo simulation.
+    It also enables the plot of the same figures but with confidence intervals.
+    """
+
     def __init__(self, path):
+        """
+        :param path: Path where with all the csv files that ends with `_statistical_result.csv`.
+            These csv are generated by cvnn.montecarlo.MonteCarlo.
+        """
         file_suffix = "_statistical_result.csv"
         super().__init__(path, file_suffix=file_suffix)
 
@@ -698,6 +756,22 @@ class MonteCarloPlotter(Plotter):
 
     def plot_line_confidence_interval(self, key='accuracy', showfig=False, savefig=True, library='matplotlib',
                                       title='', full_border=True, x_axis='epoch', extension=".svg"):
+        """
+        Saves/shows the evolution of the trainig median and mean with the corresponding confidence intervals
+            of the results.
+
+        :param library: string stating the library to be used to generate the box plot.
+            - matplotlib
+            - plotly
+            - seaborn
+        :param key: String stating what to plot using tf.keras.History labels. ex. `val_accuracy` for the validation acc
+        :param showfig: If True, it will show the grated box plot
+        :param savefig: If True, it saves the figure at: self.path / "plots/box_plot/"
+        :param title: Figure title
+        :param full_border:
+        :param x_axis: What to use for the x_axis, by default, epoch (Not recommended to change)
+        :param extension: file extensions (default svg) to be used when saving the file (ignored if library is plotly).
+        """
         if library == 'plotly':
             self._plot_line_confidance_interval_plotly(key=key, showfig=showfig, savefig=savefig,
                                                        title=title, full_border=full_border, x_axis=x_axis)
@@ -715,23 +789,40 @@ class MonteCarloPlotter(Plotter):
                            self._plot_line_confidence_interval_matplotlib.__name__ + " was called but will be omitted")
             return None
         fig, ax = plt.subplots()
-        for i, data in enumerate(self.pandas_list):
+        for i, (label, data) in enumerate(self.pandas_dict.items()):
             x = data[x_axis].unique().tolist()
             data_mean = data[data['stats'] == 'mean'][key].tolist()
-            data_max = data[data['stats'] == 'max'][key].tolist()
-            data_min = data[data['stats'] == 'min'][key].tolist()
-            data_50 = data[data['stats'] == '50%'][key].tolist()
-            data_25 = data[data['stats'] == '25%'][key].tolist()
-            data_75 = data[data['stats'] == '75%'][key].tolist()
+            if len(x) == len(data_mean):
+                data_max = data[data['stats'] == 'max'][key].tolist()
+                data_min = data[data['stats'] == 'min'][key].tolist()
+                data_50 = data[data['stats'] == '50%'][key].tolist()
+                data_25 = data[data['stats'] == '25%'][key].tolist()
+                data_75 = data[data['stats'] == '75%'][key].tolist()
+            else:
+                data_mean = []
+                data_max = []
+                data_min = []
+                data_50 = []
+                data_25 = []
+                data_75 = []
+                for ep in x:
+                    filter = [a == ep for a in data.epoch]
+                    stats = data[filter][key].describe()
+                    data_mean.append(stats['mean'])
+                    data_max.append(stats['max'])
+                    data_min.append(stats['min'])
+                    data_50.append(stats['50%'])
+                    data_25.append(stats['25%'])
+                    data_75.append(stats['75%'])
             ax.plot(x, data_mean, color=DEFAULT_MATPLOTLIB_COLORS[i],
-                    label=self.labels[i].replace('_', ' ') + ' mean')
+                    label=label.replace('_', ' ') + ' mean')
             ax.plot(x, data_50, '--', color=DEFAULT_MATPLOTLIB_COLORS[i],
-                    label=self.labels[i].replace('_', ' ') + ' median')
+                    label=label.replace('_', ' ') + ' median')
             ax.fill_between(x, data_25, data_75, color=DEFAULT_MATPLOTLIB_COLORS[i], alpha=.4,
-                            label=self.labels[i].replace('_', ' ') + ' interquartile')
+                            label=label.replace('_', ' ') + ' interquartile')
             ax.fill_between(x, data_min, data_max, color=DEFAULT_MATPLOTLIB_COLORS[i], alpha=.15,
-                            label=self.labels[i].replace('_', ' ') + ' border')
-        for label in self.labels:
+                            label=label.replace('_', ' ') + ' border')
+        for label in self.pandas_dict.keys():
             title += label.replace('_', ' ') + ' vs '
         title = title[:-3] + key
 
@@ -762,17 +853,35 @@ class MonteCarloPlotter(Plotter):
                            " was called but will be omitted")
             return None
         fig = go.Figure()
-        for i, data in enumerate(self.pandas_list):
+        for i, (label, data) in enumerate(self.pandas_dict.items()):
             # set_trace()
             x = data[x_axis].unique().tolist()
             x_rev = x[::-1]
             data_mean = data[data['stats'] == 'mean'][key].tolist()
-            data_max = data[data['stats'] == 'max'][key].tolist()
-            data_min = data[data['stats'] == 'min'][key][::-1].tolist()
-            data_50 = data[data['stats'] == '50%'][key].tolist()
-            data_25 = data[data['stats'] == '25%'][key][::-1].tolist()
-            data_75 = data[data['stats'] == '75%'][key].tolist()
-            # set_trace()
+            if len(x) == len(data_mean):
+                data_max = data[data['stats'] == 'max'][key].tolist()
+                data_min = data[data['stats'] == 'min'][key][::-1].tolist()
+                data_50 = data[data['stats'] == '50%'][key].tolist()
+                data_25 = data[data['stats'] == '25%'][key][::-1].tolist()
+                data_75 = data[data['stats'] == '75%'][key].tolist()
+            else:
+                data_mean = []
+                data_max = []
+                data_min = []
+                data_50 = []
+                data_25 = []
+                data_75 = []
+                for ep in x:
+                    filter = [a == ep for a in data.epoch]
+                    stats = data[filter][key].describe()
+                    data_mean.append(stats['mean'])
+                    data_max.append(stats['max'])
+                    data_min.append(stats['min'])
+                    data_50.append(stats['50%'])
+                    data_25.append(stats['25%'])
+                    data_75.append(stats['75%'])
+                data_min.reverse()
+                data_25.reverse()
             if full_border:
                 fig.add_trace(go.Scatter(
                     x=x + x_rev,
@@ -781,7 +890,7 @@ class MonteCarloPlotter(Plotter):
                     fillcolor=add_transparency(DEFAULT_PLOTLY_COLORS[i], 0.1),
                     line_color=add_transparency(DEFAULT_PLOTLY_COLORS[i], 0),
                     showlegend=True,
-                    name=self.labels[i].replace('_', ' ') + " borders",
+                    name=label.replace('_', ' ') + " borders",
                 ))
             fig.add_trace(go.Scatter(
                 x=x + x_rev,
@@ -790,19 +899,19 @@ class MonteCarloPlotter(Plotter):
                 fillcolor=add_transparency(DEFAULT_PLOTLY_COLORS[i], 0.2),
                 line_color=add_transparency(DEFAULT_PLOTLY_COLORS[i], 0),
                 showlegend=True,
-                name=self.labels[i].replace('_', ' ') + " interquartile",
+                name=label.replace('_', ' ') + " interquartile",
             ))
             fig.add_trace(go.Scatter(
                 x=x, y=data_mean,
                 line_color=DEFAULT_PLOTLY_COLORS[i],
-                name=self.labels[i].replace('_', ' ') + " mean",
+                name=label.replace('_', ' ') + " mean",
             ))
             fig.add_trace(go.Scatter(
                 x=x, y=data_50,
                 line=dict(color=DEFAULT_PLOTLY_COLORS[i], dash='dash'),
-                name=self.labels[i].replace('_', ' ') + " median",
+                name=label.replace('_', ' ') + " median",
             ))
-        for label in self.labels:
+        for label in self.pandas_dict.keys():
             title += label.replace('_', ' ') + ' vs '
         title = title[:-3] + key
 
@@ -826,24 +935,24 @@ class MonteCarloPlotter(Plotter):
             return None
         fig = go.Figure()
         # test plots
-        label = 'mean'
+        filter_label = 'mean'
         if median:
-            label = '50%'
-        for i, data in enumerate(self.pandas_list):
+            filter_label = '50%'
+        for i, (label, data) in enumerate(self.pandas_dict.items()):
             x = data[x_axis].unique().tolist()
-            data_mean_test = data[data['stats'] == label]["test " + key].tolist()
+            data_mean_test = data[data['stats'] == filter_label]["val_" + key].tolist()
             fig.add_trace(go.Scatter(
                 x=x, y=data_mean_test,
                 line_color=DEFAULT_PLOTLY_COLORS[i],
-                name=self.labels[i] + " test",
+                name=label + " test",
             ))
-            data_mean_train = data[data['stats'] == label]["train " + key].tolist()
+            data_mean_train = data[data['stats'] == filter_label][key].tolist()
             fig.add_trace(go.Scatter(
                 x=x, y=data_mean_train,
-                line_color=DEFAULT_PLOTLY_COLORS[i + len(self.pandas_list)],
-                name=self.labels[i].replace("_", " ") + " train ",
+                line_color=DEFAULT_PLOTLY_COLORS[i + len(self.pandas_dict.keys())],
+                name=label.replace("_", " ") + " train ",
             ))
-        title = "train and test " + key + " " + label.replace("50%", "median")
+        title = "train and test " + key + " " + filter_label.replace("50%", "median")
         fig.update_traces(mode='lines')
         fig.update_layout(title=title, xaxis_title=x_axis, yaxis_title=key)
 
@@ -851,7 +960,7 @@ class MonteCarloPlotter(Plotter):
             os.makedirs(self.path / "plots/lines/", exist_ok=True)
             plotly.offline.plot(fig,
                                 filename=str(self.path / ("plots/lines/montecarlo_" + key.replace(" ", "_")))
-                                         + "_" + label.replace("50%", "median") + ".html",
+                                         + "_" + filter_label.replace("50%", "median") + ".html",
                                 config=PLOTLY_CONFIG, auto_open=showfig)
         elif showfig:
             fig.show(config=PLOTLY_CONFIG)
@@ -859,17 +968,51 @@ class MonteCarloPlotter(Plotter):
 
 class MonteCarloAnalyzer:
 
+    """
+    This class helps organizing Monte Carlo class results and plotting them.
+    This class works with the run_data.csv generated with cvnn.montecarlo.MonteCarlo class
+    """
+
     def __init__(self, df=None, path=None):
+        """
+        There are 2 ways to use this class:
+            1. Either give the data as a pandas DataFrame.
+            2. Or give a file path to the `run_data.csv` file generated with `cvnn.montecarlo.MonteCarlo` class.
+        The class will generate the corresponding csv file (if option 1)
+            or obtain the dataframe from the csv file (option 2).
+        :param df: pandas DataFrame with the data to be plotted.
+        :param path: Optional
+            1. If df was given, this can be the a path for MonteCarloAnalyzer to save a `run_data.csv` file.
+                If path is not given, it will use the default path `./log/montecarlo/<year>/<month>/<day>/run_<time>/`
+            2. If df is not given, path should be:
+                - The full path and filename for the run_data.csv to be plotted
+                - A path to search of ALL `run_data.csv` that it can find (even within subfolders).
+                    This is useful when you want to plot together different MonteCarlo.run() results.
+                    This enables to run two simulations of 50 iterations each and plot them as if
+                        it was a single run of 100 iterations.
+        """
         self.confusion_matrix = []
         if path is not None and df is not None:  # I have data and the place where I want to save it
             self.df = df  # DataFrame with all the data
             self.path = Path(path)
             self.df.to_csv(self.path / "run_data.csv")  # Save the results for latter use
         elif path is not None and df is None:  # Load df from Path
-            if not path.endswith('.csv'):
-                path += '.csv'
-            self.df = pd.read_csv(Path(path))  # Path(__file__).parents[1].absolute() /
-            self.path = Path(os.path.split(path)[0])  # Keep only the path and not the filename
+            if not path.endswith("run_data.csv") or not path.endswith("run_data"): 
+                self.df = pd.DataFrame()
+                data_files = get_data_files_list(path)
+                for file in data_files:
+                    tmp_df = pd.read_csv(Path(file))
+                    # filter = [e == max(tmp_df.epoch) and n == 'complex_model' for e, n in
+                    #           zip(tmp_df.epoch, tmp_df.network)]
+                    # th = 0.3
+                    # if (tmp_df[filter].val_accuracy > th).all():
+                    self.df = pd.concat([self.df, tmp_df])
+                self.path = Path(path)
+            else:
+                if not path.endswith('.csv'):
+                    path += '.csv'
+                self.df = pd.read_csv(Path(path))  # Path(__file__).parents[1].absolute() /
+                self.path = Path(os.path.split(path)[0])  # Keep only the path and not the filename
         elif path is None and df is not None:  # Save df into default path
             self.path = create_folder("./log/montecarlo/")
             self.df = df  # DataFrame with all the data
@@ -918,10 +1061,16 @@ class MonteCarloAnalyzer:
     # ------------
 
     def do_all(self, extension=".svg", showfig=False, savefig=True):
-        """self.monte_carlo_plotter.plot_train_vs_test(key='loss')
-        self.monte_carlo_plotter.plot_train_vs_test(key='accuracy')
-        self.monte_carlo_plotter.plot_train_vs_test(key='loss', median=True)
-        self.monte_carlo_plotter.plot_train_vs_test(key='accuracy', median=True)"""
+        """
+        Plots box plot, histogram and confidence interval (using :code:`MonteCarloPlotter`) for both
+            1. plotly
+            2. seaborn
+        And for keys:
+            1. val_accuracy
+            2. val_loss
+            3. accuracy
+            4. loss
+        """
 
         key_list = ['accuracy', 'loss', 'val_accuracy', 'val_loss']
         for key in key_list:
@@ -934,18 +1083,29 @@ class MonteCarloAnalyzer:
                 try:
                     self.plot_histogram(key=key, library=lib, showfig=showfig, savefig=savefig, extension=extension)
                 except np.linalg.LinAlgError:
-                    logger.warning("Could not plot Histogram with " + str(lib) + " because matrix was singular",
-                                   exc_info=True)
+                    logger.warning(f"Could not plot Histogram with {lib} because matrix was singular", exc_info=True)
                 except:
                     logger.warning("Could not plot " + key + " Histogram with " + str(lib), exc_info=True)
                 try:
                     self.monte_carlo_plotter.plot_line_confidence_interval(key=key, x_axis='epoch', library=lib,
                                                                            showfig=showfig, savefig=savefig)
                 except:
-                    logger.warning("Could not plot " + key + " line_confidence_interval with " + str(lib),
-                                   exc_info=True)
+                    logger.warning(f"Could not plot {key} line_confidence_interval with {lib}", exc_info=True)
 
-    def box_plot(self, epoch=-1, library='plotly', key='val_accuracy', showfig=False, savefig=True, extension='.svg'):
+    def box_plot(self, epoch: int = -1, library: str = 'plotly', key: str = 'val_accuracy',
+                 showfig: bool = False, savefig: bool = True, extension: str = '.svg'):
+        """
+        Saves/shows a box plot of the results.
+        :param epoch: Which epoch to use for the box plot. If -1 (default) it will use the last epoch.
+        :param library: string stating the library to be used to generate the box plot.
+            - `plotly <https://plotly.com/python/>`_
+            - `seaborn <https://seaborn.pydata.org/>`_
+        :param key: String stating what to plot using tf.keras.History labels. ex. `val_accuracy` for the validation acc
+        :param showfig: If True, it will show the grated box plot
+        :param savefig: If True, it saves the figure at: self.path / "plots/box_plot/"
+        :param extension: file extensions (default svg) to be used when saving the file
+            (only used when library is seaborn).
+        """
         if library == 'plotly':
             self._box_plot_plotly(key=key, epoch=epoch, showfig=showfig, savefig=savefig)
         elif library == 'seaborn':
@@ -1113,8 +1273,22 @@ class MonteCarloAnalyzer:
                                     "plots/histogram/montecarlo_" + key.replace(" ", "_") + "_3d_histogram.html")),
                             config=PLOTLY_CONFIG, auto_open=False)
 
-    def plot_histogram(self, key='val_accuracy', epoch=-1, library='seaborn', showfig=False, savefig=True, title='',
-                       extension=".svg"):
+    def plot_histogram(self, key: str = 'val_accuracy', epoch: int = -1, library: str = 'seaborn',
+                       showfig: bool = False, savefig: bool = True, title: str = '', extension: str = ".svg"):
+        """
+        Saves/shows a histogram of the results.
+
+        :param epoch: Which epoch to use for the box plot. If -1 (default) it will use the last epoch.
+        :param library: string stating the library to be used to generate the box plot.
+            - `matplotlib <https://matplotlib.org/stable/index.html>`_
+            - `plotly <https://plotly.com/python/>`_
+            - `seaborn <https://seaborn.pydata.org/>`_
+        :param key: String stating what to plot using tf.keras.History labels. ex. `val_accuracy` for the validation acc
+        :param showfig: If True, it will show the grated box plot
+        :param savefig: If True, it saves the figure at: self.path / "plots/box_plot/"
+        :param title: Figure title
+        :param extension: file extensions (default svg) to be used when saving the file (ignored if library is plotly).
+        """
         if library == 'matplotlib':
             self._plot_histogram_matplotlib(key=key, epoch=epoch, showfig=showfig, savefig=savefig, title=title,
                                             extension=extension)
@@ -1200,25 +1374,23 @@ class MonteCarloAnalyzer:
             logger.warning("No Seaborn installed, function " + self._plot_histogram_seaborn.__name__ +
                            " was called but will be omitted")
             return None
-        fig = plt.figure()
+        fig, ax = plt.subplots()
+        ax.set_prop_cycle(color=DEFAULT_MATPLOTLIB_COLORS)
         bins = np.linspace(0, 1, 501)
         min_ax = 1.0
         max_ax = 0.0
-        ax = None
+        # ax = None
         networks_availables = self.df.network.unique()
-        # networks_availables = ['complex network', 'real network', 'polar real network']
         if epoch == -1:
             epoch = max(self.df.epoch)
-        for net in networks_availables:
-            filter = [a == net and b == epoch for a, b in zip(self.df.network, self.df.epoch)]
-            data = self.df[filter]  # Get only the data to plot
-            ax = sns.histplot(data[key], bins=bins, label=net.replace("_", " "))
-            min_ax = min(min_ax, min(data[key]))
-            max_ax = max(max_ax, max(data[key]))
+        filter = [e == epoch for e in self.df.epoch]
+        data = self.df[filter]
+        ax = sns.histplot(data, x=key, hue='network', bins=bins, legend=True)
+        min_ax = min(min_ax, min(data[key]))
+        max_ax = max(max_ax, max(data[key]))
         title += " " + key + " histogram"
-        # set_trace()
         ax.set_xlim((min_ax - 0.01, max_ax + 0.01))
-        fig.legend(loc='upper left')  # , bbox_to_anchor=(0., 0.3, 0.5, 0.5))
+        # ax.legend(loc='upper left')
         add_params(fig, ax, x_label=key.capitalize(), y_label="Occurrences",  # loc='upper left',
                    filename=self.path / (
                            "plots/histogram/montecarlo_" + key.replace(" ", "_") + "_seaborn" + extension),
@@ -1233,12 +1405,19 @@ class MonteCarloAnalyzer:
 
 
 if __name__ == "__main__":
-    path = "/home/barrachina/Documents/onera/src/PolSar/Oberpfaffenhofen/log/montecarlo/2020/10October/06Tuesday/run-16h58m53/run_data.csv"
+    path = "/home/barrachina/Documents/onera/src/PolSar/Oberpfaffenhofen/log/montecarlo/2021/02February/21Sunday/run-13h36m02"
     monte = MonteCarloAnalyzer(path=path)
-    monte.do_all(showfig=False, savefig=False)
-    monte.plot_histogram(library='matplotlib', showfig=False, savefig=False)
-    monte.monte_carlo_plotter.plot_train_vs_test(showfig=False, savefig=False)
-    monte.monte_carlo_plotter.plot_everything(showfig=False, savefig=False)
+    # monte.monte_carlo_plotter.plot_line_confidence_interval(showfig=True, library="plotly")
+    monte.plot_histogram(library="seaborn", showfig=True)
+    # monte.do_all(showfig=False, savefig=True)
+    # set_trace()
+    # monte.plot_histogram(library='matplotlib', showfig=False, savefig=True)
+    # monte.monte_carlo_plotter.plot_train_vs_test(showfig=False, savefig=True)
+    # monte.monte_carlo_plotter.plot_train_vs_test(showfig=False, savefig=True, key='loss', median=True)
+    # monte.monte_carlo_plotter.plot_train_vs_test(showfig=False, savefig=True, key='accuracy')
+    # monte.monte_carlo_plotter.plot_train_vs_test(showfig=False, savefig=True, key='accuracy', median=True)
+    # set_trace()
+    # monte.monte_carlo_plotter.plot_everything(showfig=False, savefig=True)
     """
     paths = [
         '/home/barrachina/Documents/onera/src/PolSar/Oberpfaffenhofen/log/montecarlo/2020/10October/06Tuesday/run-16h58m53/run_data',
@@ -1253,6 +1432,6 @@ if __name__ == "__main__":
     """
 
 __author__ = 'J. Agustin BARRACHINA'
-__version__ = '0.1.35'
+__version__ = '0.1.41'
 __maintainer__ = 'J. Agustin BARRACHINA'
 __email__ = 'joseagustin.barra@gmail.com; jose-agustin.barrachina@centralesupelec.fr'
